/*
IO.java
This file contains low-level file I/O routines.
*/

import java.io.*;
import java.awt.*;

class IO {
	static BufferedInputStream inStream;
	static BufferedOutputStream outStream;
	static byte[] b = new byte[4];
	static int bytesRead;
	
	//create an input stream from a given file	
	static void openForRead(File f) throws IOException {
		inStream = new BufferedInputStream(new FileInputStream(f));
		bytesRead = 0;
	}
	
	//create an output stream to a given file	
	static void openForWrite(File f) throws IOException {
		outStream = new BufferedOutputStream(new FileOutputStream(f));
	}
	
	//close current input stream
	static void closeForRead() throws IOException {
		inStream.close();
	}
	
	//flush output buffer
	static void flush() throws IOException {
		outStream.flush();
	}
	
	//close current output stream
	static void closeForWrite() throws IOException {
		flush(); //flush buffer first
		outStream.close();
	}
	
	//close current input and output streams
	static void cleanup() throws IOException {
		closeForRead();
		closeForWrite();
	}
	
	//read one byte from current input stream
	static int get() {
		int b = -1;
		try {
			b = inStream.read();
		} catch (IOException e) {
			Zip.message("Unable to read from file: " + e.getMessage());
		}
		if (b != -1) bytesRead++;
		return b;
	}
	
	//write one byte to current output stream
	static void put(int k) {
		try {
			outStream.write(k);
		} catch (IOException e) {
			Zip.message("Unable to write to file: " + e.getMessage());
		}
	}
	
	//read 4 bytes from current input stream
	static byte[] getBytes(int n) throws IOException {
		byte[] b = new byte[n];
		if (inStream.read(b) < n) throw new IOException("Unexpected end of file");
		bytesRead += n;
		return b;
	}
	
	//write a byte array to current output stream
	static void putBytes(byte[] b) throws IOException {
		outStream.write(b);
	}
	
	//get a String of length n from current input stream
	static String getString(int n) throws IOException {
		return new String(getBytes(n));
	}
	
	//write a String to current output stream
	static void putString(String s) throws IOException {
//		putBytes(s.getBytes()); //should work but doesn't
		for (int i = 0; i < s.length(); i++) {
			put((int)s.charAt(i));
		}		
	}	
	
	//get an int (4 bytes) from current input stream
	static int getInteger() throws IOException {
		int i,j,k;
		k = 0;
		for (i = 0; i < 4; i++) {
			j = get();
			if (j < 0) throw new IOException("Unexpected end of file");
			k = k*256 + j;
		}
		return k;
	}

	//write an int (4 bytes) to current output stream
	static void putInteger(int n) throws IOException {
		put((n/256/256/256)%256);
		put((n/256/256)%256);
		put((n/256)%256);
		put(n%256);
	}
	
	//copy from input stream to output stream until end of file
	//return number of bytes copied
	static int copy() throws IOException {
		int count = 0;
		int x = inStream.read();
		while (x != -1) {
			count++;
			outStream.write(x);
			x = inStream.read();
		}
		return count;
	}

	//copy from input stream to output stream
	//up to m bytes or end of file
	//return number of bytes copied
	static int copy(int m) throws IOException {
		int count,x;
		for (count = 0; count < m; count++) {
			x = inStream.read();
			if (x == -1) return count;
			outStream.write(x);
		}
		return count;
	}
}
