/*
Zip.java
This file contains the main class and all code implementing
the user interface.  The constructor initializes the user
interface.  There are also listeners to intercept various
button and menu events and dispatch the appropriate routines.
*/

import java.awt.*;
import java.awt.event.*;

public class Zip {
	static Frame frame; //main frame
	static TextArea fileList; //for displaying files in the archive
	static Label messageArea; //for displaying messages
	static MyMenuItem newMenuItem = new MyMenuItem("New Archive...");
	static MyMenuItem openMenuItem = new MyMenuItem("Open Archive...");
	static MyMenuItem closeMenuItem = new MyMenuItem("Close Archive");
	static MyMenuItem addMenuItem = new MyMenuItem("Add File...");
	static MyMenuItem extractMenuItem = new MyMenuItem("Extract...");
	static MyMenuItem byeMenuItem = new MyMenuItem("Quit");
	static MyButton newButton = new MyButton("New");
	static MyButton openButton = new MyButton("Open");
	static MyButton closeButton = new MyButton("Close");
	static MyButton addButton = new MyButton("Add");
	static MyButton extractButton = new MyButton("Extract");

	public static void main(String[] args) {
		new Zip(); //initialize user interface
		//then wait around for something to happen...
	}

	//initialize user interface
	Zip() {		
		frame = new Frame("410zip"); //main window
		//add listener for mouse events
		frame.addWindowListener(new MyWindowAdapter());
		
		//create menu bar
		MenuBar menuBar = new MenuBar();
		//file menu
		Menu fileMenu = new Menu("File");
		fileMenu.add(newMenuItem);
		fileMenu.add(openMenuItem);
		fileMenu.add(closeMenuItem);
		fileMenu.add(new MenuItem("-")); //separator
		fileMenu.add(addMenuItem);
		fileMenu.add(extractMenuItem);
		fileMenu.add(new MenuItem("-")); //separator
		fileMenu.add(byeMenuItem);
		menuBar.add(fileMenu);
		//options menu
		Menu optionsMenu = new Menu("Options");
		Menu colorMenu = new Menu("Color");
		ColorHandler.currentColor = new ColorMenuItem("White",Color.white,Color.black);
		ColorHandler.currentColor.setState(true);
		colorMenu.add(ColorHandler.currentColor);
		colorMenu.add(new ColorMenuItem("Black",Color.black,Color.lightGray));
		colorMenu.add(new ColorMenuItem("Orange",new Color(255,128,0),Color.blue));
		colorMenu.add(new ColorMenuItem("Green",Color.green,Color.black));
		colorMenu.add(new ColorMenuItem("Blue",Color.blue,Color.white));
		colorMenu.add(new ColorMenuItem("Cyan",Color.cyan,Color.black));
		colorMenu.add(new ColorMenuItem("Yellow",Color.yellow,Color.black));
		colorMenu.add(new ColorMenuItem("Purple",new Color(136,43,129),Color.white));
		optionsMenu.add(colorMenu);
		menuBar.add(optionsMenu);
		//help menu
		Menu helpMenu = new Menu("Help");
		MyMenuItem aboutMenuItem = new MyMenuItem("Get help...");
		helpMenu.add(aboutMenuItem);
		menuBar.add(helpMenu);
		frame.setMenuBar(menuBar);
		
		//create toolbar		
		Panel toolBar = new Panel(new FlowLayout(FlowLayout.LEFT));
		toolBar.setBackground(Color.lightGray);
		toolBar.add(newButton);
		toolBar.add(openButton);
		toolBar.add(closeButton);
		toolBar.add(addButton);
		toolBar.add(extractButton);
		frame.add(toolBar,"North");
		
		//create text area for file list
		fileList = new TextArea();
		fileList.setEditable(false);
		fileList.setFont(Font.decode("Courier"));
		frame.add(fileList);
		clearFiles();
		
		//create message area	
		messageArea = new Label("",Label.LEFT);
		messageArea.setBackground(Color.lightGray);
		frame.add(messageArea,"South");
		clearMessage();
		
		//display the window
		setCloseState();
		frame.setSize(500,200);
		frame.setLocation(200,200);
		frame.show();
	}
	
	//set menus and buttons for when there is an archive open
	static void setOpenState() {
		closeMenuItem.setEnabled(true);
		closeButton.setEnabled(true);
		addButton.setEnabled(true);
		addMenuItem.setEnabled(true);
		extractButton.setEnabled(true);
		extractMenuItem.setEnabled(true);
	}

	//set menus and buttons for when there is no archive open
	static void setCloseState() {
		closeMenuItem.setEnabled(false);
		closeButton.setEnabled(false);
		addButton.setEnabled(false);
		addMenuItem.setEnabled(false);
		extractButton.setEnabled(false);
		extractMenuItem.setEnabled(false);
	}

	static void setColors(Color b,Color f) {
		fileList.setBackground(b);
		fileList.setForeground(f);
	}

	static void setTitle(String s) {
		frame.setTitle(s);
	}
	
	static void listFiles(String s) {
		if (s.equals("")) s = "No files";
		fileList.setText(s);
	}
	
	static void appendFiles(String s) {
		fileList.append(s);
	}
	
	static void clearFiles() {
		fileList.setText("");
	}
	
	static void message(String s) {
		messageArea.setText(s);
	}

	static void clearMessage() {
		message("");
	}
	
	//display help dialog
	static void displayHelp() {
		Dialog helpDialog = new Dialog(Zip.frame,"410zip Help");
		helpDialog.setSize(200,200);
		helpDialog.addWindowListener(new HelpWindowAdapter());
		helpDialog.setLocation(300,300);
		helpDialog.add(new Label("Help!",Label.CENTER));
		helpDialog.show();
	}
}

/*
Various GUI objects and event handlers
*/

//handle window closing event
class MyWindowAdapter extends WindowAdapter {
	public void windowClosing(WindowEvent event) {
		System.exit(0);
	}
}

//close help window
class HelpWindowAdapter extends WindowAdapter {
	public void windowClosing(WindowEvent event) {
		event.getWindow().dispose();
	}
}

//menu items for Options menu
class ColorMenuItem extends CheckboxMenuItem {
	static ColorHandler colorHandler = new ColorHandler();
	Color backgroundColor;
	Color foregroundColor;
	
	ColorMenuItem(String s,Color b,Color f) {
		super(s);
		backgroundColor = b;
		foregroundColor = f;
		addItemListener(colorHandler);
	}
}

//handle color checkbox items
class ColorHandler implements ItemListener {
	static ColorMenuItem currentColor;
	public void itemStateChanged(ItemEvent e) {
		ColorMenuItem item = (ColorMenuItem)e.getItemSelectable();
		if (e.getStateChange() == ItemEvent.SELECTED) {
			currentColor.setState(false);
			currentColor = item;
			Zip.setColors(item.backgroundColor,item.foregroundColor);
		} else {
			item.setState(true);
		}
	}
}

//menu items
class MyMenuItem extends MenuItem {
	static MenuDispatcher dispatcher = new MenuDispatcher();
	MyMenuItem(String s) {
		super(s);
		addActionListener(dispatcher);
	}
}

//dispatch appropriate action on menu selection
class MenuDispatcher implements ActionListener {
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals("New Archive...")) {
			Archive.newArchive();
		}
		else if (command.equals("Open Archive...")) {
			Archive.openArchive();
		}
		else if (command.equals("Close Archive")) {
			Archive.archive.close();
		}
		else if (command.equals("Quit")) {
			System.exit(0);
		}
		else if (command.equals("Add File...")) {
			Archive.archive.addFile();
		}
		else if (command.equals("Extract...")) {
			Archive.archive.extract();
		}
		else if (command.equals("Get help...")) {
			Zip.displayHelp();
		}
	}
}

//buttons
class MyButton extends Button {
	static ButtonDispatcher dispatcher = new ButtonDispatcher();
	MyButton(String s) {
		super(s);
		addActionListener(dispatcher);
	}
}

//dispatch appropriate action on button event
class ButtonDispatcher implements ActionListener {
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals("New")) {
			Archive.newArchive();
		}
		else if (command.equals("Open")) {
			Archive.openArchive();
		}
		else if (command.equals("Close")) {
			Archive.archive.close();
		}
		else if (command.equals("Add")) {
			Archive.archive.addFile();
		}
		else if (command.equals("Extract")) {
			Archive.archive.extract();
		}
	}
}
